@@ -6,7 +6,7 @@ import os
 
 
 if os.getenv("WHENEVER_NO_BUILD_RUST_EXT") or (
-    platform.python_implementation() == "PyPy"
+    platform.python_implementation() in ("PyPy", "GraalVM")
 ):
     build_deps = []
 else:
