@@ -1,11 +1,11 @@
 use crate::{
     py::*,
+    tz::sync::{AtomicRefCount, SwapCell, SwapPtr, SyncCell},
     tz::tzif::{TimeZone, is_valid_key},
 };
 use ahash::AHashMap;
 use pyo3_ffi::*;
 use std::{
-    cell::UnsafeCell,
     collections::VecDeque,
     fs,
     ops::Deref,
@@ -18,22 +18,20 @@ use std::{
 /// meant to be used in a single-threaded context, it's safe to share and copy
 #[derive(Debug, Clone, Copy, Eq, PartialEq)]
 pub(crate) struct TzPtr {
-    inner: NonNull<Inner>,
+    inner: NonNull<TzRefCounted>,
 }
 
-struct Inner {
+#[derive(Debug)]
+struct TzRefCounted {
     value: TimeZone,
-    refcnt: UnsafeCell<usize>,
+    refcnt: AtomicRefCount,
 }
 
 impl TzPtr {
     /// Creates a new instance with specified refcount.
-    fn new(value: TimeZone) -> Self {
-        let inner = Box::new(Inner {
-            // We start with refcount of 2:
-            // - one to share outside of the tzstore
-            // - one to keep the pointer alive in the LRU
-            refcnt: UnsafeCell::new(2),
+    fn new(value: TimeZone, refcount: usize) -> Self {
+        let inner = Box::new(TzRefCounted {
+            refcnt: AtomicRefCount::new(refcount),
             value,
         });
         Self {
@@ -71,23 +69,21 @@ impl TzPtr {
     }
 
     fn decref(&self) -> usize {
-        unsafe {
-            let refcnt = self.inner.as_ref().refcnt.get();
-            *refcnt -= 1;
-            *refcnt
-        }
+        unsafe { self.inner.as_ref() }.refcnt.decrement()
     }
 
     fn incref(&self) {
-        unsafe {
-            let refcnt = self.inner.as_ref().refcnt.get();
-            *refcnt += 1;
-        }
+        unsafe { self.inner.as_ref() }.refcnt.increment();
+    }
+
+    /// Try to increment the refcount. Returns false if the refcount was zero
+    /// (meaning the object is being freed by another thread).
+    fn try_incref(&self) -> bool {
+        unsafe { self.inner.as_ref() }.refcnt.try_increment()
     }
 
-    #[cfg(debug_assertions)]
     pub(crate) fn ref_count(&self) -> usize {
-        unsafe { *self.inner.as_ref().refcnt.get() }
+        unsafe { self.inner.as_ref() }.refcnt.get()
     }
 
     unsafe fn drop_in_place(&self) {
@@ -150,6 +146,7 @@ impl Drop for TzHandle<'_> {
             TzHandle::NonUniquePtr(ptr) => {
                 // Non-unique pointers do not require cleanup,
                 // since they are never the last strong reference.
+                debug_assert!(ptr.ref_count() >= 2);
                 ptr.decref();
             }
         }
@@ -167,20 +164,19 @@ impl Deref for TzHandle<'_> {
     }
 }
 
-/// Timezone cache meant for single-threaded use.
-/// It's designed to be used by the ZonedDateTime class,
-/// which only calls it from a single thread while holding the GIL.
-/// This avoids the need for synchronization.
+/// Timezone cache.
+/// In GIL-enabled builds, access is synchronized by the GIL.
+/// In free-threaded builds, a mutex provides synchronization.
 /// It is based on the cache approach of zoneinfo in Python's standard library.
 #[derive(Debug)]
 struct Cache {
-    inner: UnsafeCell<CacheInner>,
+    inner: SyncCell<CacheInner>,
 }
 
 impl Cache {
     fn new() -> Self {
         Self {
-            inner: UnsafeCell::new(CacheInner {
+            inner: SyncCell::new(CacheInner {
                 lru: VecDeque::with_capacity(LRU_CAPACITY),
                 lookup: AHashMap::with_capacity(8), // a reasonable default size
             }),
@@ -189,28 +185,55 @@ impl Cache {
 
     /// Get an entry from the cache, or insert it from the supplied function.
     /// Returns a strong reference.
+    /// The load function is called outside the lock to avoid holding it during I/O.
     fn get_or_insert_with<F>(&self, key: &str, load: F) -> Option<TzPtr>
     where
         F: FnOnce() -> Option<TimeZone>,
     {
-        // SAFETY: this is safe because we only access the cache from a single thread
-        // while holding the GIL. The UnsafeCell is only used to allow mutable access
-        // to the inner cache.
-        let CacheInner { lookup, lru } = unsafe { self.inner.get().as_mut().unwrap() };
+        // First, check the cache under the lock
+        let cached = self.inner.with_mut(|CacheInner { lookup, lru }| {
+            lookup.get(key).copied().and_then(|tz| {
+                // Try to increment refcount. If it fails, the entry is being freed
+                // by another thread, so treat it as a cache miss.
+                tz.try_incref().then(|| {
+                    Self::promote_lru(tz, lru, lookup);
+                    tz
+                })
+            })
+        });
 
-        match lookup.get(key) {
-            // Found in cache. Mark it as recently used
-            Some(&tz) => {
-                tz.incref(); // Increment the refcount to ensure it's not dropped
-                Self::promote_lru(tz, lru, lookup);
-                Some(tz)
-            }
-            // Not found in cache. Load it and insert it into the cache
-            None => load().map(TzPtr::new).inspect(|&tz| {
-                Self::new_to_lru(tz, lru, lookup);
-                lookup.insert(key.to_string(), tz);
-            }),
+        if let Some(tz) = cached {
+            return Some(tz);
         }
+
+        // Cache miss: load outside the lock (may do file I/O)
+        // We start with refcount of 2:
+        // - one to return to the caller
+        // - one to keep the pointer alive in the LRU
+        let loaded = TzPtr::new(load()?, 2);
+
+        // Re-acquire lock to insert. Another thread may have loaded the same key.
+        self.inner.with_mut(|CacheInner { lookup, lru }| {
+            lookup
+                .get(key)
+                .copied()
+                .and_then(|existing| {
+                    // Another thread loaded it. Try to use theirs.
+                    existing.try_incref().then(|| {
+                        Self::promote_lru(existing, lru, lookup);
+                        // Clean up our loaded data (we don't need it)
+                        unsafe { loaded.drop_in_place() };
+                        existing
+                    })
+                })
+                .or_else(|| {
+                    // We're first (or the existing entry was stale).
+                    // Insert into cache.
+                    Self::new_to_lru(loaded, lru, lookup);
+                    lookup.insert(key.to_string(), loaded);
+                    Some(loaded)
+                })
+        })
     }
 
     fn decref<F>(tz: TzPtr, cleanup: F)
@@ -228,14 +251,14 @@ impl Cache {
     }
 
     fn remove_key(&self, key: &str) {
-        let lookup = &mut unsafe { self.inner.get().as_mut().unwrap() }.lookup;
-        lookup.remove(key);
+        self.inner.with_mut(|inner| {
+            inner.lookup.remove(key);
+        });
     }
 
     fn new_to_lru(tz: TzPtr, lru: &mut Lru, lookup: &mut Lookup) {
         debug_assert!(!lru.contains(&tz));
-        #[cfg(debug_assertions)]
-        assert!(tz.ref_count() > 0);
+        debug_assert!(tz.ref_count() > 0);
         debug_assert!(tz.key.is_some());
         // If the LRU exceeds capacity, remove the least recently used entry
         if lru.len() == LRU_CAPACITY {
@@ -260,7 +283,7 @@ impl Cache {
                 lru.push_front(tz);
             }
             None => {
-                tz.incref(); // LRU needs a strong refence
+                tz.incref(); // LRU needs a strong reference
                 Self::new_to_lru(tz, lru, lookup);
             }
         }
@@ -268,59 +291,67 @@ impl Cache {
 
     /// Clear the cache, dropping all entries.
     fn clear_all(&self) {
-        let CacheInner { lookup, lru } = unsafe { self.inner.get().as_mut().unwrap() };
+        self.inner.with_mut(|CacheInner { lookup, lru }| {
+            // Clear all weak references. Note that strong references may still exist
+            // both in the LRU and in ZonedDateTime objects.
+            lookup.clear();
 
-        // Clear all weak references. Note that strong references may still exist
-        // both in the LRU and in ZonedDateTime objects.
-        lookup.clear();
-
-        // Clear the LRU
-        let mut lru_old = std::mem::replace(lru, VecDeque::with_capacity(LRU_CAPACITY));
-        for tz in lru_old.drain(..) {
-            Self::decref(tz, || {
-                // No cleanup needed: the lookup table is already cleared
-            });
-        }
+            // Clear the LRU
+            let mut lru_old = std::mem::replace(lru, VecDeque::with_capacity(LRU_CAPACITY));
+            for tz in lru_old.drain(..) {
+                Self::decref(tz, || {
+                    // No cleanup needed: the lookup table is already cleared
+                });
+            }
+        });
     }
 
     /// Clear specific entries from the cache.
     fn clear_only(&self, keys: &[String]) {
-        let CacheInner { lookup, lru } = unsafe { self.inner.get().as_mut().unwrap() };
-        for k in keys {
-            lookup.remove(k); // Always remove, regardless of refcount
-            if let Some(i) = lru
-                .iter()
-                .position(|tz| tz.key.as_ref().expect("LRU entries always have a key") == k)
-            {
-                Self::decref(lru.remove(i).unwrap(), || {
-                    // No cleanup needed: the lookup table is already cleared
-                });
-            };
-        }
+        self.inner.with_mut(|CacheInner { lookup, lru }| {
+            for k in keys {
+                lookup.remove(k); // Always remove, regardless of refcount
+                if let Some(i) = lru
+                    .iter()
+                    .position(|tz| tz.key.as_ref().expect("LRU entries always have a key") == k)
+                {
+                    Self::decref(lru.remove(i).unwrap(), || {
+                        // No cleanup needed: the lookup table is already cleared
+                    });
+                };
+            }
+        });
     }
 }
 
 impl Drop for Cache {
-    /// Drop the cache, clearing all entries. This should only trigger during module unloading,
-    /// and there should be no ZonedDateTime objects left.
+    /// Drop the cache, clearing all entries. This should only trigger during
+    /// module unloading, and there should be no ZonedDateTime objects left.
     fn drop(&mut self) {
-        let CacheInner { lookup, lru } = unsafe { self.inner.get().as_mut().unwrap() };
-        // At this point, the only strong references should be in the LRU.
-        let mut lru = std::mem::take(lru);
-        for tz in lru.drain(..) {
-            Self::decref(tz, || {
-                // Remove the weak reference too
-                lookup.remove(tz.key.as_ref().expect("LRU entries always have a key"));
-            });
-        }
-        // By now, the lookup table should be empty (it contains only weak references)
-        debug_assert!(lookup.is_empty());
+        self.inner.with_mut(|CacheInner { lookup, lru }| {
+            // Since there are no ZonedDateTime objects left,
+            // the only strong references are in the LRU.
+            let mut lru = std::mem::take(lru);
+            for tz in lru.drain(..) {
+                debug_assert!(
+                    tz.ref_count() == 1,
+                    "Dropping cache with strong references still present"
+                );
+                Self::decref(tz, || {
+                    // Remove the weak reference too
+                    lookup.remove(tz.key.as_ref().expect("LRU entries always have a key"));
+                });
+            }
+            // By now, the lookup table should be empty (it contains only weak references)
+            debug_assert!(lookup.is_empty());
+        });
     }
 }
 
 type Lru = VecDeque<TzPtr>;
 type Lookup = AHashMap<String, TzPtr>;
 
+#[derive(Debug)]
 struct CacheInner {
     // Weak references to the timezones, keyed by TZ ID.
     // String references are held by (1) the ZonedDateTime objects, and (2) the LRU
@@ -356,10 +387,11 @@ pub(crate) struct TzStore {
     // The path to the `tzdata` Python package contents, if any.
     tzdata_path: Option<PathBuf>,
     // The paths to search for zoneinfo files. Patchable during runtime.
-    pub(crate) paths: Vec<PathBuf>,
+    paths: SwapCell<Vec<PathBuf>>,
     // We cache the system timezone here, since it's expensive to determine.
-    // The pointer represents a strong reference to a timezone.
-    system_tz_cache: UnsafeCell<Option<TzPtr>>,
+    // Uses SwapPtr for lock-free reads with atomic swaps for writes.
+    // This holds a strong reference to a TzPtr.
+    system_tz_cache: SwapPtr<TzRefCounted>,
     // This reference is borrowed from the module, which outlives this store.
     exc_notfound: PyObj,
 }
@@ -371,12 +403,17 @@ impl TzStore {
             tzdata_path: get_tzdata_path()?,
             // Empty. The actual search paths are patched in at module import
             // because this is determined in Python code.
-            paths: Vec::with_capacity(4),
-            system_tz_cache: UnsafeCell::new(None),
+            paths: SwapCell::new(Vec::with_capacity(4)),
+            system_tz_cache: SwapPtr::new(None),
             exc_notfound,
         })
     }
 
+    /// Set the timezone search paths.
+    pub(crate) fn set_paths(&self, new_paths: Vec<PathBuf>) {
+        self.paths.swap(new_paths);
+    }
+
     /// Fetches the timezone definition for the given IANA time zone ID.
     pub(crate) fn get(&self, key: &str) -> PyResult<TzHandle<'_>> {
         let ptr = self
@@ -400,31 +437,28 @@ impl TzStore {
 
     /// Retrieve the system timezone definition (cached for repeat calls).
     pub(crate) fn get_system_tz(&self) -> PyResult<TzHandle<'_>> {
-        // Check if we already have the system timezone cached
-        let ptr = match unsafe { *self.system_tz_cache.get() } {
-            Some(p) => {
-                p.incref();
-                p
+        // Fast path: check cache (lock-free read)
+        if let Some(inner) = self.system_tz_cache.load() {
+            let ptr = TzPtr { inner };
+            if ptr.try_incref() {
+                return Ok(TzHandle::Ptr(ptr, self));
             }
-            None => {
-                let p = self.determine_system_tz()?;
-                unsafe { *self.system_tz_cache.get() = Some(p) };
-                p
-            }
-        };
-        Ok(TzHandle::Ptr(ptr, self))
+            // Refcount was 0 - being freed, fall through to slow path
+        }
+        self.reset_system_tz()
     }
 
     /// Reset the cached system timezone.
-    pub(crate) fn reset_system_tz(&self) -> PyResult<()> {
-        // Clear the cached system timezone
+    pub(crate) fn reset_system_tz(&self) -> PyResult<TzHandle<'_>> {
+        // Determine new tz (does I/O)
         let new_ptr = self.determine_system_tz()?;
-        let old_ptr = unsafe { *self.system_tz_cache.get() };
-        old_ptr.inspect(|ptr| {
-            ptr.decref_with_cleanup(|| self);
-        });
-        unsafe { *self.system_tz_cache.get() = Some(new_ptr) }
-        Ok(())
+        new_ptr.incref(); // Extra ref for system_tz_cache
+
+        // Atomically swap
+        if let Some(old_inner) = self.system_tz_cache.swap(Some(new_ptr.inner)) {
+            TzPtr { inner: old_inner }.decref_with_cleanup(|| self);
+        }
+        Ok(TzHandle::Ptr(new_ptr, self))
     }
 
     /// Clear the entire cache, dropping all entries.
@@ -446,9 +480,11 @@ impl TzStore {
 
     /// Load a TZif from the TZPATH directory, assuming a benign TZ ID.
     fn load_tzif_from_tzpath(&self, key: BenignKey) -> Option<TimeZone> {
-        self.paths
-            .iter()
-            .find_map(|base| self.read_tzif_at_path(&base.join(key), Some(key)))
+        self.paths.with_read(|paths| {
+            paths
+                .iter()
+                .find_map(|base| self.read_tzif_at_path(&base.join(key), Some(key)))
+        })
     }
 
     /// Load a TZif from the tzdata package, assuming a benign TZ ID.
@@ -471,7 +507,7 @@ impl TzStore {
     }
 
     /// Get a pointer to what is currently considered the system timezone.
-    /// The pointer is already a strong reference.
+    /// The pointer represents a strong reference.
     fn determine_system_tz(&self) -> PyResult<TzPtr> {
         const ERR_MSG: &str = "get_tz() gave unexpected result";
         let tz_tuple = import(c"whenever._tz.system")?
@@ -508,7 +544,7 @@ impl TzStore {
                     .ok_or_else_raise(self.exc_notfound.as_ptr(), || {
                         format!("No time zone found at path {path:?}")
                     })?;
-                Ok(TzPtr::new(tzif))
+                Ok(TzPtr::new(tzif, 1))
             }
             // type 2: zoneinfo key OR posix TZ string (we're unsure which)
             2 => {
@@ -516,7 +552,7 @@ impl TzStore {
                     // Try to load it as a zoneinfo key first.
                     .get_or_insert_with(tz_value, || self.load_tzif(tz_value))
                     // If this fails, try to parse it as a posix TZ string.
-                    .or_else(|| TimeZone::parse_posix(tz_value).map(TzPtr::new))
+                    .or_else(|| TimeZone::parse_posix(tz_value).map(|tz| TzPtr::new(tz, 1)))
                     .ok_or_else_raise(self.exc_notfound.as_ptr(), || {
                         format!("No time zone found with key or posix TZ string {tz_value}")
                     })
@@ -533,9 +569,23 @@ impl TzStore {
 impl Drop for TzStore {
     fn drop(&mut self) {
         // Clear the system timezone cache
-        if let Some(ptr) = unsafe { *self.system_tz_cache.get() } {
+        if let Some(inner) = self.system_tz_cache.swap(None) {
+            let ptr = TzPtr { inner };
+            // When dropping, ensure the refcount is what we expect.
+            #[cfg(debug_assertions)]
+            {
+                let mut expect_refcnt = 1; // only held by system_tz_cache
+                self.cache.inner.with_mut(|CacheInner { lru, .. }| {
+                    if lru.contains(&ptr) {
+                        expect_refcnt += 1; // also held by LRU
+                    }
+                });
+                assert!(
+                    ptr.ref_count() == expect_refcnt,
+                    "Dangling references to system timezone on TzStore drop"
+                );
+            }
             ptr.decref_with_cleanup(|| self);
-            unsafe { *self.system_tz_cache.get() = None };
         }
         // The rest of the fields will be dropped automatically
     }
